import { jsxs, jsx } from '@emotion/react/jsx-runtime';
import { Text, Box, InputGroup, Input, InputRightElement, Spinner, IconButton, Icon } from '@chakra-ui/react';
import { useVariablesContext, defineComponent } from '@qorebase/app-component';
import React from 'react';
import { TargetLock } from '@feedloop/icon';
import { buildExpressionParser } from '@feedloop/expression-parser';
import { mergeConfig, javascriptConfig, evaluate } from '@feedloop/formula-editor';
import dayjs from 'dayjs';

const functions = [
    {
        name: "FormatDate",
        description: "format date with given formatting tokens",
        parameters: [
            {
                name: "format",
                type: "string",
                description: "string of formatting tokens",
            },
            { name: "date", type: "date", description: "date to format" },
        ],
        return: {
            type: "string",
            description: "string formatted date",
        },
        evaluate: (format, date) => dayjs(new Date(date)).format(format),
    },
    {
        name: "Filtrex",
        parameters: [
            { name: "expression", type: "string", description: "filtrex expression" },
        ],
        return: {
            type: "any",
            description: "evaluated expression",
        },
        evaluate: (expression) => evaluateExpression(expression),
    },
    {
        name: "FormatCurrency",
        description: "format number into currency with given locale and currency code",
        parameters: [
            { name: "number", type: "number" },
            {
                name: "locale",
                type: "string",
                description: "ISO 639-1 code with optional ISO 3166 code",
            },
            {
                name: "currency",
                type: "string",
                description: "ISO 4217 code",
            },
        ],
        examples: [
            'FormatCurrency(100000, "id", "IDR")',
            'FormatCurrency(100000, "en-US", "USD")',
        ],
        return: {
            type: "string",
            description: "string formatted currency",
        },
        evaluate: (number, locale, currency) => new Intl.NumberFormat(locale, {
            style: "currency",
            currency,
            currencyDisplay: "narrowSymbol",
        }).format(number),
    },
];
const operators = [];
const variables = [];
buildExpressionParser(mergeConfig(javascriptConfig, {
    functions,
    operators,
    variables,
}));
const evaluateExpression = (expression, additionalVariables = []) => {
    const parseExpression = buildExpressionParser(mergeConfig(javascriptConfig, {
        functions,
        operators,
        variables: variables.concat(additionalVariables),
    }));
    const parseResult = parseExpression(expression !== null && expression !== void 0 ? expression : "");
    if (parseResult.errors.length > 0)
        return {
            type: "error",
            errors: parseResult.errors,
        };
    try {
        return {
            type: "ok",
            result: evaluate(parseResult.ast, "javascript", {
                functions,
                operators,
                variables: variables.concat(additionalVariables),
            }),
        };
    }
    catch (e) {
        return { type: "ok", result: "#Error!" };
    }
};

function useDisableCondition(instance) {
    const { values } = useVariablesContext();
    const variablesConfig = React.useMemo(() => Object.entries(values).map(([name, value]) => ({
        name,
        type: "any",
        evaluate: () => value,
    })), [values]);
    const isDisabled = React.useMemo(() => {
        var _a, _b, _c;
        if (!instance ||
            !instance.disableCondition ||
            (instance.disableCondition &&
                ((_a = instance.disableCondition) === null || _a === void 0 ? void 0 : _a.conditions.length) === 0))
            return false;
        const results = (_b = instance.disableCondition) === null || _b === void 0 ? void 0 : _b.conditions.map((condition) => {
            const result = evaluateExpression(condition.formula, variablesConfig);
            return Boolean(result.type === "ok" && result.result);
        });
        return ((_c = instance.disableCondition) === null || _c === void 0 ? void 0 : _c.type) === "all"
            ? results.every((result) => result)
            : results.some((result) => result);
    }, [instance, variablesConfig]);
    return { isDisabled };
}

const InputComponentLabel = ({ title, name, isRequired, isEmpty, }) => {
    return (jsxs(Text, Object.assign({ fontSize: "lg", pb: name === "rating" ? "1" : "2", "data-test": `${name}-input-title` }, { children: [title, !isRequired && !isEmpty && (jsxs(Text, Object.assign({ display: "inline", fontStyle: "italic" }, { children: [" ", "- Optional"] })))] })));
};
var InputComponentLabel$1 = InputComponentLabel;

const TextInput = defineComponent({
    name: "Location Input",
    type: "none",
    icon: "LocationInput",
    group: "input",
    display: "inline",
    defaultProps: {
        label: "Location Input",
        hidden: false,
        placeholder: "Placeholder here",
        data: {
            inputID: "location",
            field: { type: "string", options: { format: "text" } },
            validation: {
                required: false,
            },
        },
        defaultValue: "",
    },
    propDefinition: {
        label: { group: "Design", type: "expression", options: {} },
        placeholder: {
            group: "Design",
            type: "expression",
            options: {},
        },
        hidden: { group: "Design", type: "boolean", options: {} },
        data: {
            group: "Design",
            type: "state",
            options: {
                inputID: "location",
                field: { type: "string", options: { format: "text" } },
            },
        },
        defaultValue: {
            group: "Design",
            type: "expression",
            options: {},
        },
    },
    Component: (props) => {
        var _a;
        const title = props.hooks.useEvaluate(props.properties.label);
        const placeholder = props.hooks.useEvaluate(props.properties.placeholder);
        const defaultValue = props.hooks.useEvaluate(props.properties.defaultValue);
        const data = props.hooks.useRuntimeInput(props.properties.data, props.data.page.row, {
            defaultValue,
            validate: (value) => {
                if (!value) {
                    return { error: "Required field, Please enter the data" };
                }
            },
        });
        const handleChange = React.useCallback((e) => {
            data.onChange(e.currentTarget.value);
        }, [data.onChange]);
        const [supported, setSupported] = React.useState(true);
        const [locating, setLocating] = React.useState(true);
        const getLocation = () => {
            if (navigator.geolocation) {
                setLocating(true);
                navigator.geolocation.getCurrentPosition((position) => {
                    data.onChange(`${position.coords.latitude},${position.coords.longitude}`);
                    setLocating(false);
                });
            }
            else {
                setLocating(false);
                setSupported(false);
            }
        };
        // @ts-ignore
        const { isDisabled } = useDisableCondition(props.instance);
        const disabledProperties = {
            border: "1px solid rgb(226, 232, 240)",
            pointerEvents: isDisabled ? "none" : "auto",
            background: isDisabled && "#E7E7E7",
            color: isDisabled && "#A2A2A2",
        };
        React.useEffect(() => {
            getLocation();
        }, []);
        return (jsxs(Box, Object.assign({ display: props.properties.hidden ? "none" : undefined }, { children: [jsx(InputComponentLabel$1, { title: title, name: "location", isRequired: (_a = props.properties.data.validation) === null || _a === void 0 ? void 0 : _a.required, isEmpty: title.length === 0 }), jsxs(InputGroup, Object.assign({ size: "md" }, { children: [jsx(Input, { pr: "10", style: disabledProperties, bgColor: "surface.900", placeholder: placeholder, name: props.properties.data.inputID, defaultValue: data.value, onChange: handleChange, disabled: supported, _disabled: { border: "1px solid rgb(226, 232, 240)" }, "data-test": "location-input" }, props.properties.data.inputID), jsx(InputRightElement, Object.assign({ width: "10" }, { children: locating ? (jsx(Spinner, {})) : (jsx(IconButton, { "aria-label": "locate", variant: "link", icon: jsx(Icon, { as: TargetLock }), onClick: getLocation, "data-test": "location-input-toggle", style: disabledProperties })) }))] })), !!data.validation.error && (jsx(Text, Object.assign({ fontSize: "sm", color: "red", "data-test": "location-input-error" }, { children: data.validation.error })))] })));
    },
});

export { TextInput as default };
